
----------------------------------------------------------------
-- write by xerysherry

local debug = debug;

-- 得到基类
local Object = getClass("Object");

local Rect = getClass("Object.Rect");
local Graphics = getClass("Object.Graphics");

local Mouse = getClass("Object.Mouse");
local Keyboard = getClass("Object.Keyboard");
local Joystick = getClass("Object.Joystick");

local TimerList = getClass("Object.TimerList");

local UpdateTimer = "_updater"

-----------------------------------------------------------
-- View鼠标按钮枚举
local ViewMouseEnum = class("ViewMouseEnum", Object,
{
	LB = 0, MB = 1, RB = 2,			--左，中，右键-
	WHEEL_UP = 3, WHEEL_DOWN = 4,	--滚轮，上下-
	X1 = 5, X2 = 6,					--button 4, button 5
});

-- love鼠标按钮枚举
local LoveMouseEnum = {
	LB = "l", MB = "m", RB = "r",		--左，中，右键-
	WHEEL_UP = "wu", WHEEL_DOWN = "wd",	--滚轮，上下-
	X1 = "x1", X2 = "x2",				--button 4, button 5
};

-- 鼠标事件转换表（字符串值转换为数值）
local LoveMouse2ViewMouse = 
{
	[LoveMouseEnum.LB] = ViewMouseEnum.LB,
	[LoveMouseEnum.MB] = ViewMouseEnum.MB,
	[LoveMouseEnum.RB] = ViewMouseEnum.RB,
	
	[LoveMouseEnum.WHEEL_UP] = ViewMouseEnum.WHEEL_UP,
	[LoveMouseEnum.WHEEL_DOWN] = ViewMouseEnum.WHEEL_DOWN,
	
	[LoveMouseEnum.X1] = ViewMouseEnum.X1,
	[LoveMouseEnum.X1] = ViewMouseEnum.X1,
};

-- 转换
function LoveMouseToViewMouse(button)
	return LoveMouse2ViewMouse[button];
end

-----------------------------------------------------------
-- 键盘特殊按钮枚举
local SpecialKeyEnum = class("SpecialKeyEnum", Object,
{
	_escape = 27,	-- Esc键
	_backspace = 8,	-- 退格键
	_enter = 13,	-- 回车键
	_tab = 9,		-- 制表符
	_space = 32,	-- 空格符
});

-- 检查
function SpecialKeyEnum.check(char, unicode)
	return string.byte(char) == unicode;
end

-----------------------------------------------------------
-- Gui管理类
local Gui = class("Gui", Object,
{
	-- 显示
	show = true,
	-- 激活
	enable = true,

	-- 坐标位置
	rect = nil,
	-- 焦点
	focus = nil,
	-- 前一个焦点
	prev_focus = nil,
	-- 绘图工具和画布索引
	graph = nil,
	cv = nil,
	-- 刷新率
	fps = 30,
	-- Alpha
	alpha = 255,
	-- 抓住的view
	grab_view = nil,
	-- 抓住的view的坐标偏移
	grab_x = 0,
	grab_y = 0,
	
	-- 回调对象
	mouse = nil,
	keyboard = nil,
	joystick = nil,
	timerlist = nil,
	
	-- view列表
	viewlist = nil,
	
	-- 鼠标事件
	onLeave = function () end,
	onReach = function () end,
});

-- 坐标，大小，绘图工具，画布索引
function Gui:initialize(x, y, w, h, graph, cv)
	self.rect = Rect:new(x, y, w, h);
	self.viewlist = {};
	if graph then
		self.graph = graph;
	else
		self.graph = Graphics:new();
	end
	
	if cv then
		self.cv = cv;
	else
		self.cv = "default";
	end
	
	if self.graph:getCanvas(self.cv)==nil then
		self.graph:createCanvas(self.cv, w, h);
	end
	
	self.mouse = Mouse:new();
	self.keyboard = Keyboard:new();
	self.joystick = Joystick:new();
	
	self.timerlist = TimerList:new();
	
	local timer = self:newTimer(UpdateTimer, 1/self.fps);
	timer:start();
	
	-- 鼠标事件
	self.mouse.onMove = function (x, y)
		local rx, ry = x-self.rect.x, y-self.rect.y;
		local mv = nil;
		for i,v in pairs(self.viewlist) do
			-- 检测是否在内部
			if mv==nil and v.show and IsPointIn(rx, ry, v.rect) then
				mv = v;
			else
				v:mouseleave();
			end
		end
		if mv then
			mv:mousemove(rx, ry);
		end
	end
	
	self.mouse.onPressed = function (x, y, button)
		-- 是否抓住
		local rx, ry = x-self.rect.x, y-self.rect.y;
		for i,v in pairs(self.viewlist) do
			if v.show and IsPointIn(rx, ry, v.rect) then
				v:mousepressed(rx, ry, button);
				break;
			end
		end
	end
	
	self.mouse.onReleased = function (x, y, button) 
		local rx, ry = x-self.rect.x, y-self.rect.y;
		for i,v in pairs(self.viewlist) do
			if v.show and IsPointIn(rx, ry, v.rect) then
				v:mousereleased(rx, ry, button);
				break;
			end
		end
	end
	
	self.mouse.onLeave = function ()
		for i,v in pairs(self.viewlist) do
			v:mouseleave();
		end
		self:onLeave();
	end
	
	self.mouse.onReach = function ()
		self:onReach();
	end
	
	-- 键盘事件
	self.keyboard.onChar = function (char) end;
	self.keyboard.onPressed = function (key) 
		if debug then
			if Keyboard._isDown("lctrl") then
				if key=="d" then
					debug.debug();
				end
			end
		end
	end;
	self.keyboard.onReleased = function (key) end;
	
	-- 手柄事件
	self.joystick.onPressed = function (joystick, button) end;
	self.joystick.onReleased = function (joystick, button) end;
	
	-- 定时器
	timer.onTimer = function ()
		if self.enable then
			local x, y = self.mouse._getX(), self.mouse._getY();
			-- 是否抓住
			if self.grab_view then
				self.grab_view:mousemove(self.grab_x + x - self.mouse:getLastX(),
					self.grab_y + y - self.mouse:getLastY());
			else
				if IsPointIn(x, y, self.rect) then
					self.mouse:move(x, y);
				else
					self.mouse:leave();
				end
			end
		end
	
		for i,v in pairs(self.viewlist) do
			v:onUpdate();
			break;
		end
	
		self.updateCanvas(self);
	end
end

-- 添加View(idx为nil时，将无法知道他的索引)
function Gui:add(idx, v, x, y)
	assert(v~=nil, "can not add nil!");
	if idx then
		assert(self.viewlist[idx]==nil, "This index is not nil!");
		self.viewlist[idx] = v;
	else
		table.insert(self.viewlist, v);
	end
	-- 根view，无父
	v.parent = nil;
	v.rect.x, v.rect.y = x, y;
	v:_setGui(self);
end

-- 获得子View
function Gui:get(idx)
	return self.viewlist[idx];
end

-- 删除View
function Gui:del(idx)
	local v = self.viewlist[idx];
	if v==nil then
		return;
	end
	v:clear();
	self.viewlist[idx] = nil;
end

function Gui:newTimer(idx, second)
	return self.timerlist:createTimer(idx, second);
end

function Gui:getTimer(idx)
	return self.timerlist:getTimer(idx);
end

function Gui:delTimer(idx)
	self.timerlist:delTimer(idx);
end

-- 设置刷新率
function Gui:setFPS(fps)
	self.fps = fps;
	self:getTimer(UpdateTimer):reset(1/self.fps);
end

function Gui:getPoint()
	return self.rect:getPoint();
end

function Gui:getSize()
	return self.rect:getSize();
end

function Gui:getRect()
	return self.rect:getRect();
end

-- 抓住
function Gui:grab(view, x, y)
	assert(self.grab_view==nil, "someview is grabed!");
	self.grab_view = view;
	self.grab_x = x;
	self.grab_y = y;
end

function Gui:ungrab(view)
	if self.grab_view==view then
		self.grab_view = nil;
	end
end

function Gui:showGui(s)
	if s~=nil then
		self.show = s;
	else
		self.show = true;
	end
end

function Gui:enableGui(e)
	if s~=nil then
		self.enable = e;
	else
		self.enable = true;
	end
end 

-- 设置焦点
function Gui:setFocus(view)
	self.pre_focus = self.focus;
	self.focus = view;
	
	if self.focus then
		self.focus:onSetFocus(self.pre_focus);
	end
	if self.pre_focus then
		self.pre_focus:onKillFocus(self.focus);
	end
end

----------------------------------------------------------------
-- 对应于love各个回调
function Gui:mousepressed(x, y, button)
	if not self.enable or not self.show then
		return false;
	end

	if IsPointIn(x, y, self.rect) then
		self.mouse:pressed(x, y, LoveMouseToViewMouse(button));
		return true;
	end
	return false;
end

function Gui:mousereleased(x, y, button)
	if not self.enable or not self.show then
		return false;
	end

	-- 是否抓住
	if self.grab_view then
		self.grab_view:mousereleased(self.grab_x + x - self.mouse:getLastX(),
			self.grab_y + y - self.mouse:getLastY(), LoveMouseToViewMouse(button));
		return true;
	end
	
	if IsPointIn(x, y, self.rect) then
		self.mouse:released(x, y, LoveMouseToViewMouse(button));
		return true;
	end
	return false;
end

function Gui:keypressed(key, unicode)
	if not self.enable then
		return;
	end

	self.keyboard:pressed(key, unicode);
	if self.focus then
		self.focus:keypressed(key, unicode);
	end
end

function Gui:keyreleased(key)
	if not self.enable then
		return;
	end
	
	self.keyboard:released(key);
	if self.focus then
		self.focus:keyreleased(key);
	end
end

function Gui:joystickpressed(joystick, button)
	if not self.enable then
		return;
	end
	
	self.joystick:pressed(joystick, button);
end

function Gui:joystickreleased(joystick, button)
	if not self.enable then
		return;
	end
	
	self.joystick:released(joystick, button);
end

function Gui:update(dt)
	self.timerlist:tic(dt);
end

function Gui:updateCanvas()
	if not self.show then
		return;
	end

	-- 设置绘制目标
	self.graph:setCanvas(self.cv);

	-- 设置裁剪区
	Graphics._setScissor(0, 0, self.rect.width, self.rect.height);
	
	-- 清理
	Graphics._setColor({0,0,0,0});
	Graphics._setColorMode("replace");
	Graphics._setBlendMode("alpha")
	Graphics.rectangle("fill", self.rect);
	Graphics._setColorMode("modulate");
	
	-- 测试用边框
	-- Graphics._setColor({255,0,0});
	-- Graphics._rectangle("line", 0,0,
		-- self.rect.width, self.rect.height);
	
	for i,v in pairs(self.viewlist) do
		v:update();
		v:draw(0, 0, 
			-- 裁剪区
			0, 0, self.rect.width, self.rect.height);
	end
	-- 取消裁剪区
	Graphics._setScissor();
	
	-- 恢复绘制到屏幕
	self.graph:setCanvas();
	
end

function Gui:draw()
	if not self.show then
		return;
	end
	
	-- 绘制
	Graphics._setColor({255,255,255,self.alpha});
	self.graph:drawCanvas(self.cv, self.rect.x, self.rect.y);
end

return Gui;
